/* Copyright (c) 2014, Florian Jung
   All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met: 

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer. 
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution. 

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* IMPORTANT
 * after each think-cycle or another cube-deleting cycle
 * should be checked if the rows need further work or not
 * (if not, this requires another full-row cycle)
 *
 * on program init, all numbered rows need needswork=1,
 * all non-numbered need =0
 */

 /* todo: fclose */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define MAX_NUM 10
#define OPTIMIZE_STACK
/* #define RECURSE */
/* #define RECURSION_DEPTH 1 */
/* #define MEASURE_RECURSION */
#define MEASURE_TIME

#define max(a,b) ( (a)>(b) ? (a) : (b) )

#define MODE_XY 1
#define MODE_YZ 2        
#define MODE_XZ 3
#define TYPE_NORMAL 0
#define TYPE_CIRCLE 1
#define TYPE_SQUARE 2

typedef struct
{
  int n;
  int marked;
  int target;
  int typ;
  int needswork;
} arr_t;


#define cube(_x,_y,_z) (_cube[(_z) + (_y)*zlen + (_x)*zlen*ylen])
#define xy(_x,_y) _xy[(_y) + (_x)*ylen]
#define yz(_y,_z) _yz[(_z) + (_y)*zlen]
#define xz(_x,_z) _xz[(_z) + (_x)*zlen]

int xlen,ylen,zlen;
int *_cube;
arr_t *_xy,*_xz,*_yz;
#if defined(RECURSION_DEPTH) || defined(MEASURE_RECURSION)
int rec=0;
#ifdef MEASURE_RECURSION
int maxrec=0;
#endif
#ifdef RECURSION_DEPTH
int again;
#endif
#endif

#ifdef OPTIMIZE_STACK
  int mode,len;
  int nmarks;
  int j;
  int cnt,cnttmp;
  int init_cumulate;
  int didsth=0;
	int tmpmark[MAX_NUM];
	int *tmprow;
#endif

FILE *infile, *outfile;


int rm_cube(int x, int y, int z)
{
  if (cube(x,y,z)==1)
  {
/*		printf("rm_cube %i %i %i\n",x,y,z);*/
		cube(x,y,z)=0;
    xy(x,y).n--;
    yz(y,z).n--;
    xz(x,z).n--;
    return 1;
	}
  return 0;
}

int mark_cube(int x, int y, int z)
{
  if (cube(x,y,z)==1)
  {
/*		printf("mark_cube %i %i %i\n",x,y,z); */
		cube(x,y,z)=2;
    xy(x,y).marked++;
    yz(y,z).marked++;
    xz(x,z).marked++;
    return 1;
  }
  return 0;
}



void think(int x, int y, int z)
{
  arr_t *a;
  int *cumulate;
	int i;

#ifndef OPTIMIZE_STACK
  int mode,len;
  int nmarks;
  int j;
  int cnt,cnttmp;
  int init_cumulate;
  int didsth=0;
	int tmpmark[MAX_NUM];
	int *tmprow;
#endif


  if (x==-1)      {mode=MODE_YZ; a=&yz(y,z);}
  else if (y==-1) {mode=MODE_XZ; a=&xz(x,z);}
  else if (z==-1) {mode=MODE_XY; a=&xy(x,y);}
	else {printf("think: one coord. must be -1!\n"); return;}

#if defined(RECURSION_DEPTH) || defined(MEASURE_RECURSION)
	rec++;
#ifdef RECURSION_DEPTH
	if (rec>RECURSION_DEPTH)
	{
		again=1;
		rec--;
		return;
	}
#endif
	/* printf("recursion depth=%i\n",rec); */
#ifdef MEASURE_RECURSION
	if (rec>maxrec)
	{
		maxrec=rec;
		printf("max recursion depth = %i\n",maxrec);
	}
#endif
#endif

  if (a->needswork)
  {
    if (a->target!=a->marked)  /* non-full row? */
    {
      switch (mode) /* init tmprow, and len */
      {
        case MODE_XY:
          for (i=0;i<zlen;i++)
            tmprow[i]=cube(x,y,i);
          len=zlen;
          break;
        case MODE_XZ:
          for (i=0;i<ylen;i++)
            tmprow[i]=cube(x,i,z);
          len=ylen;
          break;
        case MODE_YZ:
          for (i=0;i<xlen;i++)
            tmprow[i]=cube(i,y,z);
          len=xlen;
          break;
      }
			cumulate=(int*) malloc(len*sizeof(*cumulate));

      nmarks=a->target-a->marked;
			for (i=0;i<nmarks;i++)
        tmpmark[i]=i;

      i=nmarks-1;
      tmpmark[i]--; /* to correct the first ++ */
      init_cumulate=1;
			while(1)
      {
head:
        tmpmark[i]++;
        while(tmpmark[i]>=len)
        {
          i--;
          if (i==-1) goto done;
          tmpmark[i]++;
        }
        while(i<nmarks-1)
        {
          i++;
					tmpmark[i]=tmpmark[i-1]+1;
        }
        if (tmpmark[i]>=len) goto head;

        for (j=0;j<nmarks;j++)                  /* invalid solution, */
          if (tmprow[tmpmark[j]]!=1) goto head; /* try next */

        /* let's try that solution */
        for (j=0;j<nmarks;j++)
          tmprow[tmpmark[j]]=2;

        /* count n_groups */
				cnt=0; cnttmp=0;
        for (j=0;j<len;j++)
          if (tmprow[j]==2)
          {
            if (cnttmp==0) { cnttmp=1; cnt++; }
          }
          else
            cnttmp=0;

        /* as many groups as expected :) */
				if ( ((a->typ==TYPE_NORMAL) && (cnt==1)) ||
             ((a->typ==TYPE_CIRCLE) && (cnt==2)) ||
             ((a->typ==TYPE_SQUARE) && (cnt>=3)) )
        {
          if (init_cumulate)
          {
            for (j=0;j<len;j++)
							cumulate[j]= ((tmprow[j]==2) ? 2 : 0);
            init_cumulate=0;
          }
          else
          {
            for (j=0;j<len;j++)
							if (cumulate[j]!=((tmprow[j]==2) ? 2 : 0))
                cumulate[j]=1;
          }
        }

        /* undo temporary changes */
        for (j=0;j<nmarks;j++)
          tmprow[tmpmark[j]]=1;
      }
done:
			if (!init_cumulate)
			{
				switch(mode) /* apply changes */
				{
					case MODE_XY:
						for (i=0;i<zlen;i++)
							if (cumulate[i]==2)
							{
								if (mark_cube(x,y,i))
								{
									didsth=1;
									#ifdef RECURSE
										think(x,-1,i);
										think(-1,y,i);
									#endif
								}
							}
							else if (cumulate[i]==0)
							{
								if (rm_cube(x,y,i))
								{
									didsth=1;
									#ifdef RECURSE
										think(x,-1,i);
										think(-1,y,i);
									#endif
								}
							}
						break;

					case MODE_XZ:
						for (i=0;i<ylen;i++)
							if (cumulate[i]==2)
							{
								if (mark_cube(x,i,z))
								{
									didsth=1;
									#ifdef RECURSE
										think(x,i,-1);
										think(-1,i,z);
									#endif
								}
							}
							else if (cumulate[i]==0)
							{
								if (rm_cube(x,i,z))
								{
									didsth=1;
									#ifdef RECURSE
										think(x,i,-1);
										think(-1,i,z);
									#endif
								}
							}
						break;

					case MODE_YZ:
						for (i=0;i<xlen;i++)
							if (cumulate[i]==2)
							{
								if (mark_cube(i,y,z))
								{
									didsth=1;
									#ifdef RECURSE
										think(i,y,-1);
										think(i,-1,z);
									#endif
								}
							}
							else if (cumulate[i]==0)
							{
								if (rm_cube(i,y,z))
								{
									didsth=1;
									#ifdef RECURSE
										think(i,y,-1);
										think(i,-1,z);
									#endif
								}
							}
						break;
				}
				if ((a->target==a->marked) && (a->n==a->marked))
					a->needswork=0;
			}

			free(cumulate);
    }
		else /* if ((a->target!=a->n) /* && (a->target==a->marked)) /* full row? */ */ */
    {
      a->needswork=0;
      switch (mode)  /* remove all unmarked blocks */
      {              /* in the row */
				case MODE_XY:
					for (i=0;i<zlen;i++)
            if (rm_cube(x,y,i))
            {
							didsth=1;
              #ifdef RECURSE
								think(x,-1,i);
								think(-1,y,i);
							#endif
            }
          break;
        case MODE_XZ:
					for (i=0;i<ylen;i++)
            if (rm_cube(x,i,z))
            {
							didsth=1;
              #ifdef RECURSE
								think(x,i,-1);
								think(-1,i,z);
							#endif
            }
          break;
        case MODE_YZ:
					for (i=0;i<xlen;i++)
            if (rm_cube(i,y,z))
            {
							didsth=1;
              #ifdef RECURSE
								think(i,y,-1);
								think(i,-1,z);
							#endif
            }
          break;
      }
    }
	}
#if defined(RECURSION_DEPTH) || defined(MEASURE_RECURSION)
	rec--;
#endif
}


char mygetc(FILE *f)
{
	char c;
	if (f==0)
		return getch();
	else
	{
		do
			c=fgetc(f);
		while ((c==' ') || (c=='\n'));
		if (c==EOF)
		{
			printf("error: unexpected EOF\n");
			exit(1);
		}
		if (c=='x') c=' '; /* files use x for space */
		return c;
	}
}

void _input(arr_t *p, arr_t **oldp)
{
	char c;
	while(1)
	{
		c=mygetc(infile);
		if (c==' ')
		{
			p->target=-1;
			p->needswork=0;
			if (outfile) fputc('x',outfile);
			break;
		}
		else if ((c>='0') && (c<='9'))
		{
			p->target=c-'0';
			p->needswork=1;
			p->typ=TYPE_NORMAL;
			*oldp=p;
			if (outfile) fputc(c,outfile);
			break;
		}
		else if ((c>='a') && (c<='n'))
		{
			p->target=c-'a'+10;
			p->needswork=1;
			p->typ=TYPE_NORMAL;
			*oldp=p;
      if (outfile) fputc(c,outfile);
			break;
		}
		else if (c=='o')
		{
			(*oldp)->typ=TYPE_CIRCLE;
      if (outfile) fputc(c,outfile);
		}
		else if ((c=='#') || (c=='p'))
		{
			(*oldp)->typ=TYPE_SQUARE;
      if (outfile) fputc(c,outfile);
		}
	}
}

void input()
{
	int x,y,z;
	arr_t *p, *oldp,t;

	printf("\nenter front view\n");
  if (outfile) fputc('\n',outfile);
	oldp=&xy(0,0);
	for (y=0;y<ylen;y++)
	{
		for (x=0;x<xlen;x++)
		{
			p=&xy(x,y);
			_input(p,&oldp);
			printf("%02i  ",p->target);
		}
		printf("\n");
		if (outfile) fputc('\n',outfile);
	}
	_input(&t,&oldp);


	printf("\nenter right side view\n");
  if (outfile) fputc('\n',outfile);
	oldp=&yz(0,0);
	for (y=0;y<ylen;y++)
	{
		for (z=0;z<zlen;z++)
		{
			p=&yz(y,z);
			_input(p,&oldp);
			printf("%02i  ",p->target);
		}
		printf("\n");
    if (outfile) fputc('\n',outfile);
	}
	_input(&t,&oldp);


	printf("\nenter front top view\n");
  if (outfile) fputc('\n',outfile);
	oldp=&xz(0,zlen-1);
	for (z=zlen-1;z>=0;z--)
	{
		for (x=0;x<xlen;x++)
		{
			p=&xz(x,z);
			_input(p,&oldp);
			printf("%02i  ",p->target);
		}
		printf("\n");
    if (outfile) fputc('\n',outfile);
	}
	_input(&t,&oldp);
}


int main(int argc, char **argv)
{
	int i,ret,x,y,z;
	int count=0;
	char buf[10];
#ifdef MEASURE_TIME
	clock_t begin;
#endif

	infile=0; outfile=0;

	if (argc>=2)
	{
		infile=fopen(argv[1],"r");
		if (infile==0)
		{
			outfile=fopen(argv[1],"w");
			if (outfile==0)
				printf("opening file '%s' for writing failed: %s\n",argv[1],strerror(errno));
			else
				printf("opening file '%s' for output\n",argv[1]);
		}
		else
			printf("opening file '%s' for input\n",argv[1]);
	}


	if (infile==0)
	{
		printf("xlen=");	gets(buf);	xlen=atoi(buf);
		printf("ylen=");	gets(buf);	ylen=atoi(buf);
		printf("zlen=");	gets(buf);	zlen=atoi(buf);

		if (outfile)
		{
			fprintf(outfile,"%i\n%i\n%i\n\n",xlen,ylen,zlen);
		}
	}
	else
	{
		fgets(buf,sizeof(buf),infile);  xlen=atoi(buf);
		fgets(buf,sizeof(buf),infile);  ylen=atoi(buf);
		fgets(buf,sizeof(buf),infile);  zlen=atoi(buf);
		printf("xlen=%i\nylen=%i\nzlen=%i\n",xlen,ylen,zlen);
	}


	_cube=(int*) malloc(xlen*ylen*zlen*sizeof(*_cube));
	_xy=(int*) malloc(xlen*ylen*sizeof(*_xy));
	_xz=(int*) malloc(xlen*zlen*sizeof(*_xz));
	_yz=(int*) malloc(ylen*zlen*sizeof(*_yz));


  #ifdef OPTIMIZE_STACK
		tmprow=(int*) malloc(max(max(xlen,ylen),zlen)*sizeof(*tmprow));
  #endif

	input();

	for(i=0;i<xlen*ylen;i++)
		_xy[i].marked=_xy[i].n=0;

	for(i=0;i<xlen*zlen;i++)
		_xz[i].marked=_xz[i].n=0;

	for(i=0;i<ylen*zlen;i++)
		_yz[i].marked=_yz[i].n=0;


  for (i=0;i<xlen*ylen*zlen;i++)
    _cube[i]=1;

	/* start calculation */
#ifdef MEASURE_TIME
	begin=clock();
#endif

#ifndef RECURSE
	do
	{
		didsth=0;
		count++;
#else
#ifdef RECURSION_DEPTH
	do
	{
		again=0;
		count++;
#endif
#endif
		for (x=0;x<xlen;x++)
		{
			for(y=0;y<ylen;y++)
				think(x,y,-1);

			for (z=0;z<zlen;z++)
				think(x,-1,z);
		}
		for(y=0;y<ylen;y++)
			for (z=0;z<zlen;z++)
				think(-1,y,z);
#ifndef RECURSE
	} while(didsth);
#else
#ifdef RECURSION_DEPTH
	} while(again);
#endif
#endif

#ifdef MEASURE_TIME
	printf("calculation took %li ticks / %f msec of processing time\n",(clock()-begin),(float)(clock()-begin)*1000.0/CLK_TCK);
#endif
	printf("solved in %i loops\n",count);
	/* done */

	/* display result */
	for (z=0;z<zlen;z++)
	{
		printf("\n\n\n");
		for (y=0;y<ylen;y++)
		{
			for (x=0;x<xlen;x++)
				printf ("%i  ",cube(x,y,z));
			printf("\n");
		}
		getch();
	}
}
